!SFX_LIC Copyright 1994-2014 CNRS, Meteo-France and Universite Paul Sabatier
!SFX_LIC This is part of the SURFEX software governed by the CeCILL-C licence
!SFX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt  
!SFX_LIC for details. version 1.
!     #########
SUBROUTINE VEGTYPE_GRID_TO_PATCH_GRID(KPATCH,KNPATCH,PVEGTYPE_PATCH,PPATCH,KMASK,PFIELDOUT,PW)
!        ################################################
!!
!!****  *VEGTYPE_GRID_TO_PATCH_GRID* averages fields from all (12) vegtypes 
!!                                   on only a few patches
!!    PURPOSE
!!    -------
!
!              
!!**  METHOD
!!    ------
!!
!!    REFERENCE
!!    ---------
!!
!!      
!!    AUTHOR
!!    ------
!!
!!      V. Masson          * Meteo-France *
!!
!!    MODIFICATIONS
!!    -------------
!!      Original    01/2004
!!
!-------------------------------------------------------------------------------

!
USE MODD_SURF_PAR,       ONLY : XUNDEF
USE MODD_DATA_COVER_PAR, ONLY : NVEGTYPE
!
USE MODI_VEGTYPE_TO_PATCH
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
!
!*      0.1    declarations of arguments
!
INTEGER, INTENT(IN) :: KPATCH
INTEGER, INTENT(IN) :: KNPATCH
REAL, DIMENSION(:,:), INTENT(IN)   :: PVEGTYPE_PATCH
REAL, DIMENSION(:), INTENT(IN)   :: PPATCH
INTEGER, DIMENSION(:), INTENT(IN) :: KMASK
REAL, DIMENSION(:,:,:), INTENT(IN) :: PFIELDOUT
REAL, DIMENSION(:,:), INTENT(OUT) :: PW
!
!
!*      0.2    declarations of local variables
!
REAL :: ZINT, ZINT2, ZINT3
REAL :: ZPREC, ZPREC2
REAL :: ZWI
REAL, DIMENSION(SIZE(PW,1),SIZE(PW,2)) :: ZW0
!
REAL, DIMENSION(SIZE(PW,1)) :: ZSUM
INTEGER                       :: JP, JD   ! loop on patches
INTEGER                       :: JVEG  ! loop on vegtypes
INTEGER                       :: JL, JI, IMASK    ! loop on layers
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
!---------------------------------------------------------------------------
!
!* averages from vegtypes to chosen number of patches
IF (LHOOK) CALL DR_HOOK('VEGTYPE_GRID_TO_PATCH_GRID',0,ZHOOK_HANDLE)
!
ZSUM(:) = 0.
DO JVEG=1,NVEGTYPE
  JP = VEGTYPE_TO_PATCH(JVEG,KNPATCH)
  IF (JP/=KPATCH) CYCLE
  DO JI = 1,SIZE(PW,1)
    ZSUM(JI) =ZSUM(JI) + PVEGTYPE_PATCH(JI,JVEG)   
  ENDDO
ENDDO
!
PW(:,:) = 0.
!
ZW0(:,:) = 0.
DO JVEG=1,NVEGTYPE
  JP = VEGTYPE_TO_PATCH(JVEG,KNPATCH)
  IF (JP/=KPATCH) CYCLE
  DO JL=1,SIZE(PW,2)
    DO JI = 1,SIZE(PW,1)
      IMASK = KMASK(JI)
      ZWI = PVEGTYPE_PATCH(JI,JVEG) 
      ZWI = ZWI * PFIELDOUT(IMASK,JL,JVEG)
      ZW0(JI,JL) = ZW0(JI,JL) + ZWI      
      !PW(JI,JL) = PW(JI,JL) + PVEGTYPE_PATCH(JI,JVEG) * PFIELDOUT(IMASK,JL,JVEG)
    ENDDO
  END DO
END DO
!
PW(:,:) = ZW0(:,:)
!
DO JI = 1,SIZE(PW,1)
  IF (ZSUM(JI)/=0.) PW(JI,:) = PW(JI,:) / ZSUM(JI)
ENDDO
!
ZPREC = 1.0E+16
!
DO JP = 1,SIZE(PW,2)
  DO JI = 1,SIZE(PW,1)
    IF (PW(JI,JP)/=XUNDEF) THEN
      ZPREC2 = 1.
      DO JD = 0,15
        IF (ABS(PW(JI,JP))<1./(10**(JD))) THEN
          ZPREC2 = ZPREC2*10.
        ELSE
          EXIT
        ENDIF
      ENDDO   
      ZINT = AINT(PW(JI,JP))
      ZINT2 = AINT((PW(JI,JP)-ZINT)*ZPREC)
      ZINT3 = ANINT((PW(JI,JP)-ZINT-ZINT2/ZPREC)*ZPREC*ZPREC2)      
      IF (PW(JI,JP)/=ZINT) THEN
        PW(JI,JP) = ZINT + ZINT2/ZPREC + ZINT3/ZPREC/ZPREC2
      ENDIF
    ENDIF
  ENDDO
ENDDO
!
!* insures undefined value when patch is not present
!
DO JP=1,KNPATCH
  DO JL=1,SIZE(PW,2)
    WHERE(PPATCH(:)==0.) PW(:,JL) = XUNDEF
  END DO
END DO
WHERE( ABS(PW-XUNDEF)/XUNDEF < 1.E-6 ) PW = XUNDEF
IF (LHOOK) CALL DR_HOOK('VEGTYPE_GRID_TO_PATCH_GRID',1,ZHOOK_HANDLE)
!
!---------------------------------------------------------------------------
!
END SUBROUTINE VEGTYPE_GRID_TO_PATCH_GRID
